# Makefile for Bluetooth server (btserver).
#
# Copyright (C) 2004-2007 TomTom BV <http://www.tomtom.com/>

TOPDIR:=../../../../
include $(TOPDIR)/Linux/common-linux.mk

# Build tools
CC=$(TT_CC)
CXX=$(TT_CXX)
LD=$(TT_LD)
AR=$(TT_AR)
MKDIR=$(TT_MKDIR)
RMDIR=$(TT_RMDIR)
clndir=$(TT_clndir)
RM=$(TT_RM)
CP=$(TT_CP)
SED=$(TT_SED)

# Paths to dependencies
# TODO Always use staging
ifeq ($(NASHVILLE_SUPPORT),y)
BLUEZINC=$(TOPDIR)/$(STAGING)/include
BLUEZLIB=$(TOPDIR)/$(STAGING)/lib
else
BLUEZINC=$(TOPDIR)/External/bluez/2.15/Include
BLUEZLIB=$(TOPDIR)/External/bluez/2.15/Linux/lib-$(PLATFORM)
endif
TTMPDIR=$(TOPDIR)/TTMP
COMPDIR=$(TOPDIR)/components

# All include directories
INCLUDES=\
-I. \
-I$(TOPDIR)/GameFrame/bluetooth \
-I$(COMPDIR)/Bluetooth/btserver/interfaces \
-I$(BLUEZINC) \
-I$(TTMPDIR) \
-I$(TTMPDIR)/Linux \
-I$(TOPDIR)/OsAbstractionLayer/linux \
-I$(TOPDIR)/GameFrame/FRAMECOMMON/LINUX

# Preprocessor, C/C++ compiler and linker flags
CPPFLAGS=$(TT_CPPFLAGS) $(INCLUDES)
CFLAGS=$(TT_CFLAGS) $(TT_WARNFLAGS)
ifeq ($(NASHVILLE_SUPPORT),y)
CPPFLAGS+=-DNASHVILLE_BLUETOOTH
CFLAGS+=-DNASHVILLE_BLUETOOTH
endif
CXXFLAGS=$(TT_CXXFLAGS) $(TT_WARNFLAGS)
LDFLAGS=$(TT_LDFLAGS)

# Extra libraries
TTMPLIB=$(TTMPDIR)/lib-$(TT_OUT)/libttmp.so
LDLIBS=-L$(BLUEZLIB) -lbluetooth -L$(TTMPDIR)/lib-$(TT_OUT) -lttmp

# Output directories
OBJDIR=$(CURDIR)/obj-$(TT_OUT)
BINDIR=$(CURDIR)/bin-$(TT_OUT)

# Target
TARGET=$(BINDIR)/btserver

# Objects
OBJECTS=\
$(OBJDIR)/btserver.o \
$(OBJDIR)/bthandler.o \
$(OBJDIR)/btsocket.o \
$(OBJDIR)/btutils.o \
$(OBJDIR)/hidd.o \
$(OBJDIR)/sdp.o \
$(OBJDIR)/bthidremote.o

# Generic rules
$(OBJDIR)/%.o: %.c
	$(tt-compile-c)

$(OBJDIR)/%.o: %.cpp
	$(tt-compile-cpp)

# Specific rules
.PHONY: all clean depclean allclean

all: $(TARGET)

$(TARGET): $(OBJECTS) $(TTMPLIB)
	@$(MKDIR) $(BINDIR)
	$(LINK.o) $^ $(LDLIBS) -o $@

$(TTMPLIB):
	$(MAKE) -C $(TTMPDIR)

clean:
	-$(RM) $(TARGET) $(OBJECTS)
	@$(call clndir,$(BINDIR) $(OBJDIR))

# Dependencies
ALL_DEPS=$(OBJECTS:.o=.d)

depclean:
	-$(RM) $(ALL_DEPS)
	@$(call clndir,$(BINDIR) $(OBJDIR))

allclean: clean depclean

ifndef NO_DEPS_GEN
-include $(ALL_DEPS)
endif

# EOF
