#ifndef _BTSOCKET_H_
#define _BTSOCKET_H_

#include <netdb.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/rfcomm.h>
#include <bluetooth/l2cap.h>

#include "btserverdefines.h"

/* do not intend to support multiple ttys for now, pppd always connects to /dev/rfcomm0 */
#define BTS_TTY_TEMPL	"/dev/rfcomm%d"
#define BTS_TTY_NAME_LEN	13


/* to be used by pppd */
#define BTS_TTY_DEFAULT_ID      0
/* socket readiness */
#define BTS_TTY_NOTREADY	0
#define BTS_TTY_READY		1

#define L2CAP_PSM_AVCTP 23

struct sBTSocket {
	bdaddr_t bdaddr;
	int chan;
	int sock;
	int accsock;
        int tty_id;
	char tty[BTS_TTY_NAME_LEN];
};
typedef struct sBTSocket *BTSocket;
typedef const struct sBTSocket *cBTSocket;

int BTSocketCompare(cBTSocket sk, const char* bdaddr, int chan, int *res);

int BTSocketCreate(BTSocket* sk, const char* bdaddr, int chan);
int BTSocketDestroy(BTSocket sk);

int BTSocketListen(BTSocket sk, int chan, int profile);
int BTSocketStopListen(BTSocket sk);
int BTSocketAccept(BTSocket sk, int timeout, int profile);

int BTSocketConnect(BTSocket sk, int timeout, int profile);
int BTSocketIsConnected(cBTSocket sk, int timeout);

int BTSocketSend(cBTSocket sk, const char* buf, int len, int *sent, int timeout);
int BTSocketReceive(cBTSocket sk, char *buf, int len, int *received, int timeout);

int BTSocketClose(BTSocket sk);

int BTSocketCreateTTY(BTSocket sk, int tty_id);
int BTSocketIsTTYReady(cBTSocket sk, int *ready);
int BTSocketReleaseTTY(BTSocket sk);
int BTSocketGetTTYName(cBTSocket sk, char *ttyname);

int BTSocketGetAddress(cBTSocket sk, char *addr);
int BTSocketGetChannel(cBTSocket sk, int *chan);

int BTSocketGetMtuSize(cBTSocket sk, int * imtu, int * omtu);

#endif //_BTSOCKET_H_
