#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <dirent.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/rfcomm.h>
#include <errno.h>
#include "btserverdefines.h"
#include "nonbluez.h"
#include "btutils.h"

#ifdef ENABLE_DEBUG_LOG
char* state2str(int state){
  switch (state) {
  case BT_STATE_UNINITIALIZED: return "BT_STATE_UNINITIALIZED";
  case BT_STATE_INITIALIZED: return "BT_STATE_INITIALIZED";
  case BT_STATE_START_LISTENING: return "BT_STATE_START_LISTENING";
  case BT_STATE_LISTENING: return "BT_STATE_LISTENING";
  case BT_STATE_START_CONNECTING: return "BT_STATE_START_CONNECTING";
  case BT_STATE_REMOVE_CONNECTION: return "BT_STATE_REMOVE_CONNCETION";
  case BT_STATE_CONNECTING: return "BT_STATE_CONNECTING";
  case BT_STATE_CHECK_PAIRING: return "BT_STATE_CHECK_PAIRING";
  case BT_STATE_CONNECTED: return "BT_STATE_CONNECTED";
  case BT_STATE_APP_START: return "BT_STATE_APP_START";
  case BT_STATE_APP_REMOVE_CONNECTION: return "BT_STATE_APP_REMOVE_CONNECTION";
  case BT_STATE_APP_READY: return "BT_STATE_APP_READY";
  case BT_STATE_APP_CLOSING: return "BT_STATE_APP_CLOSING";
  case BT_STATE_CLOSING: return "BT_STATE_CLOSING";
  case BT_PPP_STATE_UNINITIALIZED: return "BT_PPP_STATE_UNINITIALIZED";
  case BT_PPP_STATE_INIT: return "BT_PPP_STATE_INIT";
  case BT_PPP_STATE_REMOVE: return "BT_PPP_STATE_REMOVE";
  case BT_PPP_STATE_CREATETTY: return "BT_PPP_STATE_CREATETTY";
  case BT_PPP_STATE_CHECKTTY: return "BT_PPP_STATE_CHECKTTY";
  case BT_PPP_STATE_START: return "BT_PPP_STATE_START";
  case BT_PPP_STATE_CHECKREADY: return "BT_PPP_STATE_CHECKREADY";
  case BT_PPP_STATE_CLOSING: return "BT_PPP_STATE_CLOSING";
  case BT_STATE_RETRIEVE_CHANNEL: return "BT_STATE_RETRIEVE_CHANNEL";
  case BT_STATE_RETRIEVING_CHANNEL: return "BT_STATE_RETRIEVING_CHANNEL";
  case BT_PPP_RETRIEVE_CHANNEL: return "BT_PPP_RETRIEVE_CHANNEL";
  case BT_PPP_RETRIEVING_CHANNEL: return "BT_PPP_RETRIEVING_CHANNEL";
  case BT_PPP_STATE_HCITOOLCHECK: return "BT_PPP_STATE_HCITOOLCHECK";
  case BT_PPP_STATE_HCITOOLCHECKREADY: return "BT_PPP_STATE_HCITOOLCHECKREADY";

  default: 
 	DEBUG_LOG1("UNKNOWN STATE (%d)",state);
 	return "!UNKNOWN!";

  }
}
char* error2str(int error){
  switch (error) {
  case BT_ERR_NONE: return "BT_ERR_NONE";
  case BT_ERR_FAILED: return "BT_ERR_FAILED";
  case BT_ERR_TIMEOUT: return "BT_ERR_TIMEOUT";
  case BT_ERR_ARGUMENT: return "BT_ERR_ARGUMENT";
  case BT_ERR_NOMEMORY: return "BT_ERR_NOMEMORY";
  case BT_ERR_STATE: return "BT_ERR_STATE";
  case BT_ERR_INUSE: return "BT_ERR_INUSE";
  case BT_ERR_CONNECTION: return "BT_ERR_CONNECTION";
  default: return "!UNKNOWN!";
  }
}
#endif //ENABLE_DEBUG_LOG

int BTSendSig(char* procname, int sig)
{
  struct dirent *de;
  char  path[PATH_MAX + 1];
  char  name[PATH_MAX + 1];
  DIR   *dir;
  int   pid = -1;

  dir = opendir(BTU_PROCBASE);
  if (!dir) {
    DEBUG_LOG1("[BT UTILS] BTSendSig(): cannot locate %s\n", BTU_PROCBASE);
    return -1;
  }
  while ((de = readdir(dir)) != NULL) {
    if (isdigit(de->d_name[0])) {
      sprintf(path, "%s/%s/exe", BTU_PROCBASE, de->d_name);
      memset(name, 0, PATH_MAX + 1);
      readlink(path, name, PATH_MAX);
      if ((strlen(name) == strlen(procname) && !strcmp(name, procname)) ||
          (strlen(name) > strlen(procname) && name[strlen(name) - strlen(procname) - 1] == '/' && !strcmp(&name[strlen(name) - strlen(procname)], procname)) )
        pid = atoi(de->d_name);
    }
  }
  closedir(dir);
  if (sig > 0) {
    if (pid > 0) {
      DEBUG_LOG3("[BT UTILS] BTSendSig(): sending %d to %s(%d)\n", sig, procname, pid);
      kill(pid, sig);
    } else {
      DEBUG_LOG("[BT UTILS] BTSendSig(): cannot locate pid\n");
    }
  } else {
    DEBUG_LOG2("[BT UTILS] BTSendSig(): located %s at pid %d\n", procname, pid);
  }

  return pid;
}

void BTSetTimer(BTTimer timer, unsigned int timeout)
{
  struct timeval tvadd;

  gettimeofday(&timer->start, NULL);
  if (timeout < 1000000) {
    tvadd.tv_sec = 0;
    tvadd.tv_usec = timeout;
  } else {
    tvadd.tv_sec = timeout / 1000000;
    tvadd.tv_usec = timeout % 1000000;
  }
  timeradd(&timer->start, &tvadd, &timer->stop);
  timer->timeout = timeout;
}

int BTTimerExpired(BTTimer timer)
{
  struct timeval now;

  gettimeofday(&now, NULL);
  if (timercmp(&now, &timer->start, <= )) {
    DEBUG_LOG("[BT UTILS] BTTimerExpired : adjusting a timer\n");
    BTSetTimer(timer, now.tv_sec*1000000 + now.tv_usec + timer->timeout / 2);
  }
  return timercmp(&timer->stop, &now, <= );
}

void BTEnablePageScan()
{
  int s;
  struct THCIDeviceRequirements dr;

  if ((s = socket(AF_BLUETOOTH, SOCK_RAW, BluetoothProtocolHCI)) != -1) {
    dr.iDeviceID  = 0;
    dr.iDeviceOptions = HciDeviceScanPage | HciDeviceScanInquiry;
    if (fcntl(s, F_SETFL, O_NONBLOCK) != -1) {
      if (ioctl(s, HCISETSCAN, (unsigned long)&dr) != -1) {
        DEBUG_LOG("[BT UTILS] BTEnablePageScan : page scan enabled\n");
      } else {
        DEBUG_LOG("[BT UTILS] BTEnablePageScan : enabling page scan failed\n");
      }
    } else {
      DEBUG_LOG("[BT UTILS] BTEnablePageScan : enabling page scan fcntl failed\n");
    }
    close(s);
  } else {
    DEBUG_LOG("[BT UTILS] BTEnablePageScan : enabling page scan socket failed\n");
  }
}

void BTDisablePageScan()
{
  int s;
  struct THCIDeviceRequirements dr;

  if ((s = socket(AF_BLUETOOTH, SOCK_RAW, BluetoothProtocolHCI)) != -1) {
    if (fcntl(s, F_SETFL, O_NONBLOCK) != -1) {
      dr.iDeviceID  = 0;
      dr.iDeviceOptions = HciDeviceDisableScan;
      if ((ioctl(s, HCISETSCAN, (unsigned long)&dr)) != -1) {
        DEBUG_LOG("[BT UTILS] BTDisablePageScan : page scan disabled\n");
      } else {
        DEBUG_LOG("[BT UTILS] BTDisablePageScan : disabling page scan ioctl failed\n");
      }
    } else {
      DEBUG_LOG("[BT UTILS] BTDisablePageScan : disabling page scan fcntl failed\n");
    }
    close(s);
  } else {
    DEBUG_LOG("[BT UTILS] BTDisablePageScan : disabling page scan socket failed\n");
  }
}

int BTGetActiveAclDev(char *bdaddrstr)
{
  struct hci_dev_list_req *dl = NULL;
  struct hci_dev_req *dr;
  struct hci_conn_list_req *cl = NULL;
  struct hci_conn_info *ci;
  bdaddr_t bdaddr;
  int ii, i, sk;
  int result = -1;

  if ((sk = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0)
  {
    DEBUG_LOG("[BT UTILS] : BTGetActiveAclDev BTPROTO_HCI socket failed.\n");
    return -1;
  }
  if (!(dl = (struct hci_dev_list_req*)malloc(HCI_MAX_DEV * sizeof(*dr) + sizeof(*dl))))
  {
    DEBUG_LOG("[BT UTILS] : BTGetActiveAclDev hci_dev_list_req malloc failed.\n");
    goto done;
  }
  if (!(cl = (struct hci_conn_list_req*)malloc(10 * sizeof(*ci) + sizeof(*cl))))
  {
    DEBUG_LOG("[BT UTILS] : BTGetActiveAclDev hci_dev_list_req malloc failed.\n");
    goto done;
  }
  dl->dev_num = HCI_MAX_DEV;
  dr = dl->dev_req;
  if (ioctl(sk, HCIGETDEVLIST, (void *) dl)) {
    DEBUG_LOG("[BT UTILS] : BTGetActiveAclDev can't get device list\n");
    goto done;
  }

  for (i = 0; i < dl->dev_num; i++, dr++)	{
    if (hci_test_bit(HCI_UP, &dr->dev_opt)) {
      cl->dev_id = dr->dev_id;
      cl->conn_num = 10;
      ci = cl->conn_info;
      if (ioctl(sk, HCIGETCONNLIST, (void *) cl)) {
        DEBUG_LOG("[BT UTILS] : BTGetActiveAclDev can't get connection list\n");
        goto done;
      }
      for (ii = 0; ii < cl->conn_num; ii++, ci++) {
        str2ba(bdaddrstr, &bdaddr);
        if ( !bacmp(&ci->bdaddr, &bdaddr) && ci->type == ACL_LINK) {
          //DEBUG_LOG("[BT UTILS] : BTGetActiveAclDev found ACL link \n");
          result = dr->dev_id;
          goto done;
        }
      }
    }
  }

done:
  close(sk);
  if (cl)
    free(cl);
  if (dl)
    free(dl);
  return result;
}

int BTIsAclLinkActive(char *bdaddrstr)
{
  if (BTGetActiveAclDev(bdaddrstr) < 0)
    return BT_FALSE;
  else
    return BT_TRUE;
}

void BTDestroyAclLink(char *bdaddrstr)
{
  struct hci_conn_info_req *cr = 0;
  bdaddr_t bdaddr;
  int dd = -1;
  int dev_id = BTGetActiveAclDev(bdaddrstr);

  if (dev_id < 0)
    goto failed;

  dd = hci_open_dev(dev_id);
  if (dd < 0) {
    DEBUG_LOG("[BT UTILS] : BTDestroyAclLink HCI device open failed \n");
    goto failed;
  }

  if (!(cr = (struct hci_conn_info_req*)malloc(sizeof(*cr) + sizeof(struct hci_conn_info)))) {
    DEBUG_LOG("[BT UTILS] : BTDestroyAclLink cannot allocate memory\n");
    goto failed;
  }
  str2ba(bdaddrstr, &bdaddr);
  bacpy(&cr->bdaddr, &bdaddr);
  cr->type = ACL_LINK;
  if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
    DEBUG_LOG("[BT UTILS] : BTDestroyAclLink get connection info failed\n");
    goto failed;
  }

  if (hci_disconnect(dd, htobs(cr->conn_info->handle), HCI_OE_USER_ENDED_CONNECTION, 10000) < 0)
    DEBUG_LOG("[BT UTILS] : BTDestroyAclLink disconnect failed\n");

  close(dd);
  dd = 0;
  
failed:
  if (dd>0){
    close(dd);
    dd = 0;
  }  
  if (cr){
    free(cr);
    cr =0;
    }
    
  return;
}

int BTExitSniffMode(char *bdaddrstr)
{ 
  int ret = BT_ERR_NONE;
  struct hci_conn_info_req *cr;
  bdaddr_t bdaddr;
  int dd, dev_id = BTGetActiveAclDev(bdaddrstr);
  if (dev_id < 0)
    return BT_ERR_FAILED;

  dd = hci_open_dev(dev_id);
  if (dd < 0) {
    DEBUG_LOG("HCI device open failed");
    return BT_ERR_FAILED;
  }

  if (!(cr = (struct hci_conn_info_req*)malloc(sizeof(*cr) + sizeof(struct hci_conn_info)))) {
    DEBUG_LOG("cannot allocate memory\n");
    close(dd);
    dd = 0;
    return BT_ERR_FAILED;
  }
  str2ba(bdaddrstr, &bdaddr);
  bacpy(&cr->bdaddr, &bdaddr);
  cr->type = ACL_LINK;
  if (ioctl(dd, HCIGETCONNINFO, (unsigned long) cr) < 0) {
    DEBUG_LOG("get connection info failed");
    close(dd);
    dd = 0;
    free(cr);
    cr=0;
    return BT_ERR_FAILED;
  }

  if (hci_exit_sniff_mode(dd, htobs(cr->conn_info->handle), 10000) < 0)
  {
    DEBUG_LOG("hci_exit_sniff_mode failed");
    ret = BT_ERR_FAILED;
  }
  else 
  { 
    DEBUG_LOG("hci_exit_sniff_mode succeded");
    ret = BT_ERR_NONE;  
  }  
  
  close(dd);
  dd = 0;
  free(cr);
  cr = 0;

  return ret;
}


int BTGetFileInt(const char* name)
{
  int ret;
  char fname[128];

  snprintf(fname, sizeof fname, "%s", name);
  FILE *fp = fopen(fname, "r");
  if (fp != NULL)
  {
    if (fscanf(fp, "%d", &ret) != 1)
    {
      DEBUG_LOG2("%s: Unable to read value from %s\n", __func__, fname);
      ret = -1;
    }
    fclose(fp);
  }
  else
  {
    DEBUG_LOG3("%s: Unable to open %s: %s\n", __func__, fname, strerror(errno));
    ret = -1;
  }

  return ret;
}

int BTSetName(char* name)
{

  int dd, dev_id = hci_get_route(NULL), res = BT_ERR_NONE;
  if (dev_id < 0)
  {
    DEBUG_LOG("[BT UTILS] : BTSetName dev_id < 0 \n");
    return BT_ERR_FAILED;
  }
  dd = hci_open_dev(dev_id);
  if (dd < 0) {
    DEBUG_LOG("[BT UTILS] : BTSetName HCI device open failed\n");
    return BT_ERR_FAILED;
  }
  if (hci_write_local_name(dd, name, 2000)) {
    DEBUG_LOG("[BT UTILS] : BTSetName Cant read local device name\n");
    res = BT_ERR_FAILED;
  }
  close(dd);
  dd = 0;
  {
    char cmd[512];
    if (strlen(name) < 512 - strlen("sed -i -r 's/name \".*\";/name \"\";/' /etc/bluetooth/hcid.conf")) {
      sprintf(cmd, "sed -i -r 's/name \".*\";/name \"%s\";/' /etc/bluetooth/hcid.conf", name);
      system(cmd);
      DEBUG_LOG("[BT UTILS] : BTSetName ready with hcid.conf file, kill HCID ???\n");
      system("killall -HUP hcid");
      DEBUG_LOG("[BT UTILS] : BTSetName killed HCID\n");
    }
  }
  return res;
}

int BTGetName(char* name, int size)
{
  int dd, dev_id = hci_get_route(NULL), res = BT_ERR_NONE;
  if (dev_id < 0)
  {
    DEBUG_LOG("[BT UTILS] : BTGetName dev_id < 0 \n");
    return BT_ERR_FAILED;
  }
  dd = hci_open_dev(dev_id);
  if (dd < 0) {
    DEBUG_LOG("[BT UTILS] : BTGetName HCI device open failed\n");
    return BT_ERR_FAILED;
  }
  if (hci_read_local_name(dd, size, name, 1000) < 0) {
    DEBUG_LOG("[BT UTILS] : BTGetName Cant change local device name\n");
    res = BT_ERR_FAILED;
  }
  close(dd);
  dd = 0;

  return res;
}

int BtGetNameConnectedDevice(char* bdaddrstr, char *name, int len)
{
    bdaddr_t bdaddr;
    int res = BT_ERR_NONE;
    int dd = hci_open_dev(0);
  
    if (dd < 0) {
        DEBUG_LOG("HCI device open failed");
        return BT_ERR_FAILED;
	  }
    str2ba(bdaddrstr, &bdaddr);
    if (hci_read_remote_name(dd, &bdaddr, len, name, 1000) < 0)
    {
        DEBUG_LOG("Can't read remote device name\n");
        res = BT_ERR_FAILED;
    }
    DEBUG_LOG1("BtGetNameConnectedDevice returns %s \n",name);
    close(dd);
    dd = 0;
    return res;
}

struct link_key {
  bdaddr_t sba;
  bdaddr_t dba;
  uint8_t  key[16];
  uint8_t  type;
  time_t   time;
};

#ifndef NASHVILLE_BLUETOOTH
int BTIsPaired(char *bdaddrstr)
{
  static struct link_key k;
  int f, r = 0;
  bdaddr_t bdaddr;
  str2ba(bdaddrstr, &bdaddr);
  f = open(BTU_LINKKEY_FILE, O_RDONLY);
  if( f == -1)
  {
    DEBUG_LOG3("[BT UTILS] BTIsPaired : Unable to open %s. %s (%d)\n", BTU_LINKKEY_FILE, strerror(errno), errno);
    return 0;
  }
  
  while ((r = read(f, &k, sizeof(k)))) {
    if (r < 0)
    {
      close(f);
      DEBUG_LOG("[BT UTILS] BTIsPaired : read error \n");
      return 0;
    }
    if (!bacmp(&k.dba, &bdaddr))
    {
      close(f);
      DEBUG_LOG("[BT UTILS] BTIsPaired : found linkkey for this BT address\n");
      return 1;
    }
  }
  close(f);
  DEBUG_LOG("[BT UTILS] BTIsPaired : NOT found linkkey for this BT address\n");
  return 0;
}
#else
int BTIsPaired(char *bdaddrstr)
{	
	char line[100];
	char address[20];
	int found = 0;
	
	FILE *file  = fopen(BTU_LINKKEY_FILE, "r");
	
	if (file)
	{
  	   while(( fgets(line, sizeof(line), file) != NULL ) &&
	          (found==0)) 
	   {
	       sscanf(line, "%s ", address); 
	       if (!strcmp(bdaddrstr, address))
	         found = 1;
  	   }
	   fclose(file);
	}
	else
	{
	    DEBUG_LOG("[BT UTILS] BTIsPaired: linkkeys file could not be opened\n");
	}
	if (found==1) 
  {
    DEBUG_LOG("[BT UTILS] BTIsPaired: linkkey found\n");
	}
  else 
  {  
    DEBUG_LOG("[BT UTILS] BTIsPaired : linkkey NOT found\n");
	}
  return found;
}
#endif
