#ifndef _BTCLIENT_H_
#define _BTCLIENT_H_

#define BTCL_VER_MAJOR		2
#define BTCL_VER_MINOR		1

#define BTCL_UPDATE_INTERVAL	100 //ms (influenced: GetState(), GetError(), Receive())
#define BTCL_DEFAULT_PIN 	"0000"

#include "btserverdefines.h"
#include "btclientinterface.h"
#include "btttmp.h"


class BtClient : public IBtClient
{
public:

	BtClient();
	virtual ~BtClient();
	virtual int Construct();
	void Free();
	virtual int Init( char* aBdaddr, int aChannel, int aProfile, int aFlags = 0, char* aPin = 0 );
	virtual int StartUsingChannel( int aHandle, const char* aData );
	virtual int StopUsingChannel( int aHandle);
	virtual int Release( int aHandle );
	virtual int Close( int aHandle, int aForced );
	virtual int Send(int aHandle, char* aBuffer, int aSize );
	virtual int Receive(int aHandle, char* aBuffer, int* aSize);
	virtual int GetState( int aHandle );
	virtual int GetError( int aHandle );
	virtual int SetFlags( int aHandle, int aFlags );
	virtual int GetFlags( int aHandle );
	virtual int IsProfileActive( int aProfile );
	virtual int RequestConnection();
	virtual int IsConnectionAllowed( int aHandle );
	virtual void ReleaseConnectionRequest( int aHandle );
	virtual int GetBTName( char *aName, int aSize );
	virtual int SetBTName( char *aName );
	virtual int GetHidRequest(char *aAddress, int aSize);
	virtual int ConnectToHid(char *aAddress, int size);
	virtual int DisconnectHid();
	virtual int StartHid();
	virtual int GetMtuSize( int aHandle, int * imtu, int * omtu );
	virtual int CreateTTY( int aHandle, int aTtyId);
	virtual int ReleaseTTY( int aHandle);
	virtual int Reconnect(int aHandle);
	virtual int ExitSniffMode(char* aBdaddr);
	virtual int GetBTConnectionInfo(int aHandle, char* aAddr, int aSize);

private:
	int GetInternalState( int aHandle );

	int stateCache[ BT_MAX_HANDLES ];
	int errorCache[ BT_MAX_HANDLES ];
	int receiveCache[ BT_MAX_HANDLES ];
	TTicks lastUpdatedReceive[ BT_MAX_HANDLES ];
	TTicks lastUpdatedState[ BT_MAX_HANDLES ];
	TTicks lastUpdatedError[ BT_MAX_HANDLES ];
	TTMPConnection iConnection;

	TTMPMessage iMessage;
	int iCmdId;
};

#endif //_BTCLIENT_H_
